@@ -1,23 +1,32 @@
 # -*- test-case-name: twisted.web.test.test_web -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Implementation of the lowest-level Resource class.
 """
 
+import warnings
+
 from zope.interface import Attribute, implements, Interface
 
+from twisted.web import http
+
 
 class IResource(Interface):
-    """A web resource."""
+    """
+    A web resource.
+    """
 
-    isLeaf = Attribute(\
-"""Signal if this IResource implementor is a "leaf node" or not. If True,
-getChildWithDefault will not be called on this Resource.""")
+    isLeaf = Attribute(
+        """
+        Signal if this IResource implementor is a "leaf node" or not. If True,
+        getChildWithDefault will not be called on this Resource.
+        """)
 
     def getChildWithDefault(name, request):
-        """Return a child with the given name for the given request.
+        """
+        Return a child with the given name for the given request.
         This is the external interface used by the Resource publishing
         machinery. If implementing IResource without subclassing
         Resource, it must be provided. However, if subclassing Resource,
@@ -25,11 +34,13 @@ getChildWithDefault will not be called on this Resource.""")
         """
 
     def putChild(path, child):
-        """Put a child IResource implementor at the given path.
+        """
+        Put a child IResource implementor at the given path.
         """
 
     def render(request):
-        """Render a request. This is called on the leaf resource for
+        """
+        Render a request. This is called on the leaf resource for
         a request. Render must return either a string, which will
         be sent to the browser as the HTML for the request, or
         server.NOT_DONE_YET. If NOT_DONE_YET is returned,
@@ -39,8 +50,11 @@ getChildWithDefault will not be called on this Resource.""")
         """
 
 
+
 def getChildForRequest(resource, request):
-    """Traverse resource tree to find who will handle the request."""
+    """
+    Traverse resource tree to find who will handle the request.
+    """
     while request.postpath and not resource.isLeaf:
         pathElement = request.postpath.pop(0)
         request.prepath.append(pathElement)
@@ -48,8 +62,10 @@ def getChildForRequest(resource, request):
     return resource
 
 
+
 class Resource:
-    """I define a web-accessible resource.
+    """
+    I define a web-accessible resource.
 
     I serve 2 main purposes; one is to provide a standard representation for
     what HTTP specification calls an 'entity', and the other is to provide an
@@ -107,7 +123,8 @@ class Resource:
     # Concrete HTTP interface
 
     def getChild(self, path, request):
-        """Retrieve a 'child' resource from me.
+        """
+        Retrieve a 'child' resource from me.
 
         Implement this to create dynamic resource generation -- resources which
         are always available may be registered with self.putChild().
@@ -128,14 +145,12 @@ class Resource:
         @param request: a twisted.web.server.Request specifying meta-information
                         about the request that is being made for this child.
         """
-        # This is ugly, I know, but since error.py directly accesses
-        # resource.Resource during import-time (it subclasses it), the Resource
-        # class must be defined by the time error is imported.
-        from twisted.web import error
-        return error.NoResource("No such child resource.")
+        return NoResource("No such child resource.")
+
 
     def getChildWithDefault(self, path, request):
-        """Retrieve a static or dynamically generated child resource from me.
+        """
+        Retrieve a static or dynamically generated child resource from me.
 
         First checks if a resource was added manually by putChild, and then
         call getChild to check for dynamic resources. Only override if you want
@@ -145,18 +160,19 @@ class Resource:
         This will check to see if I have a pre-registered child resource of the
         given name, and call getChild if I do not.
         """
-        if self.children.has_key(path):
+        if path in self.children:
             return self.children[path]
-
         return self.getChild(path, request)
 
+
     def getChildForRequest(self, request):
-        import warnings
         warnings.warn("Please use module level getChildForRequest.", DeprecationWarning, 2)
         return getChildForRequest(self, request)
 
+
     def putChild(self, path, child):
-        """Register a static child.
+        """
+        Register a static child.
 
         You almost certainly don't want '/' in your path. If you
         intended to have the root of a folder, e.g. /foo/, you want
@@ -165,8 +181,10 @@ class Resource:
         self.children[path] = child
         child.server = self.server
 
+
     def render(self, request):
-        """Render a given resource. See L{IResource}'s render method.
+        """
+        Render a given resource. See L{IResource}'s render method.
 
         I delegate to methods of self with the form 'render_METHOD'
         where METHOD is the HTTP that was used to make the
@@ -185,10 +203,13 @@ class Resource:
         """
         m = getattr(self, 'render_' + request.method, None)
         if not m:
-            from twisted.web.server import UnsupportedMethod
+            # This needs to be here until the deprecated subclasses of the
+            # below three error resources in twisted.web.error are removed.
+            from twisted.web.error import UnsupportedMethod
             raise UnsupportedMethod(getattr(self, 'allowedMethods', ()))
         return m(request)
 
+
     def render_HEAD(self, request):
         """
         Default handling of HEAD method.
@@ -197,3 +218,83 @@ class Resource:
         the framework will handle this correctly.
         """
         return self.render_GET(request)
+
+
+
+class ErrorPage(Resource):
+    """
+    L{ErrorPage} is a resource which responds with a particular
+    (parameterized) status and a body consisting of HTML containing some
+    descriptive text.  This is useful for rendering simple error pages.
+
+    @ivar template: A C{str} which will have a dictionary interpolated into
+        it to generate the response body.  The dictionary has the following
+        keys:
+
+          - C{"code"}: The status code passed to L{ErrorPage.__init__}.
+          - C{"brief"}: The brief description passed to L{ErrorPage.__init__}.
+          - C{"detail"}: The detailed description passed to
+            L{ErrorPage.__init__}.
+
+    @ivar code: An integer status code which will be used for the response.
+    @ivar brief: A short string which will be included in the response body.
+    @ivar detail: A longer string which will be included in the response body.
+    """
+
+    template = """
+<html>
+  <head><title>%(code)s - %(brief)s</title></head>
+  <body>
+    <h1>%(brief)s</h1>
+    <p>%(detail)s</p>
+  </body>
+</html>
+"""
+
+    def __init__(self, status, brief, detail):
+        Resource.__init__(self)
+        self.code = status
+        self.brief = brief
+        self.detail = detail
+
+
+    def render(self, request):
+        request.setResponseCode(self.code)
+        request.setHeader("content-type", "text/html")
+        return self.template % dict(
+            code=self.code,
+            brief=self.brief,
+            detail=self.detail)
+
+
+    def getChild(self, chnam, request):
+        return self
+
+
+
+class NoResource(ErrorPage):
+    """
+    L{NoResource} is a specialization of L{ErrorPage} which returns the HTTP
+    response code I{NOT FOUND}.
+    """
+    def __init__(self, message="Sorry. No luck finding that resource."):
+        ErrorPage.__init__(self, http.NOT_FOUND,
+                           "No Such Resource",
+                           message)
+
+
+
+class ForbiddenResource(ErrorPage):
+    """
+    L{ForbiddenResource} is a specialization of L{ErrorPage} which returns the
+    I{FORBIDDEN} HTTP response code.
+    """
+    def __init__(self, message="Sorry, resource is forbidden."):
+        ErrorPage.__init__(self, http.FORBIDDEN,
+                           "Forbidden Resource",
+                           message)
+
+
+__all__ = [
+    'IResource', 'getChildForRequest',
+    'Resource', 'ErrorPage', 'NoResource', 'ForbiddenResource']
