@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.web.test.test_webclient -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,11 +9,13 @@ HTTP client.
 import os, types
 from urlparse import urlunparse
 
+from twisted.python import log
 from twisted.web import http
 from twisted.internet import defer, protocol, reactor
 from twisted.python import failure
 from twisted.python.util import InsensitiveDict
 from twisted.web import error
+from twisted.web.http_headers import Headers
 from twisted.python.compat import set
 
 
@@ -26,9 +28,16 @@ class PartialDownloadError(error.Error):
 
 
 class HTTPPageGetter(http.HTTPClient):
+    """
+    Gets a resource via HTTP, then quits.
+
+    Typically used with L{HTTPClientFactory}.  Note that this class does not, by
+    itself, do anything with the response.  If you want to download a resource
+    into a file, use L{HTTPPageDownloader} instead.
+    """
 
     quietLoss = 0
-    followRedirect = 1
+    followRedirect = True
     failed = 0
 
     _specialHeaders = set(('host', 'user-agent', 'cookie', 'content-length'))
@@ -60,8 +69,18 @@ class HTTPPageGetter(http.HTTPClient):
             self.transport.write(data)
 
     def handleHeader(self, key, value):
+        """
+        Called every time a header is received. Stores the header information
+        as key-value pairs in the C{headers} attribute.
+
+        @type key: C{str}
+        @param key: An HTTP header field name.
+
+        @type value: C{str}
+        @param value: An HTTP header field value.
+        """
         key = key.lower()
-        l = self.headers[key] = self.headers.get(key, [])
+        l = self.headers.setdefault(key, [])
         l.append(value)
 
     def handleStatus(self, version, status, message):
@@ -122,7 +141,11 @@ class HTTPPageGetter(http.HTTPClient):
         self.quietLoss = True
         self.transport.loseConnection()
 
-    handleStatus_302 = lambda self: self.handleStatus_301()
+    def handleStatus_302(self):
+        if self.afterFoundGet:
+            self.handleStatus_303()
+        self.handleStatus_301()
+
 
     def handleStatus_303(self):
         self.factory.method = 'GET'
@@ -141,7 +164,7 @@ class HTTPPageGetter(http.HTTPClient):
                 failure.Failure(
                     error.Error(
                         self.status, self.message, response)))
-        if self.factory.method.upper() == 'HEAD':
+        if self.factory.method == 'HEAD':
             # Callback with empty string, since there is never a response
             # body for HEAD requests.
             self.factory.page('')
@@ -178,6 +201,11 @@ class HTTPPageDownloader(HTTPPageGetter):
             self.factory.pagePart(data)
 
     def handleResponseEnd(self):
+        if self.length:
+            self.transmittingPage = 0
+            self.factory.noPage(
+                failure.Failure(
+                    PartialDownloadError(self.status)))
         if self.transmittingPage:
             self.factory.pageEnd()
             self.transmittingPage = 0
@@ -220,6 +248,11 @@ class HTTPClientFactory(protocol.ClientFactory):
     @ivar redirectLimit: The maximum number of HTTP redirects that can occur
           before it is assumed that the redirection is endless.
 
+    @type afterFoundGet: C{bool}
+    @ivar afterFoundGet: Deviate from the HTTP 1.1 RFC by handling redirects
+        the same way as most web browsers; if the request method is POST and a
+        302 status is encountered, the redirect is followed with a GET method
+
     @type _redirectCount: int
     @ivar _redirectCount: The current number of HTTP redirects encountered.
     """
@@ -234,13 +267,14 @@ class HTTPClientFactory(protocol.ClientFactory):
 
     def __init__(self, url, method='GET', postdata=None, headers=None,
                  agent="Twisted PageGetter", timeout=0, cookies=None,
-                 followRedirect=1, redirectLimit=20):
-        self.protocol.followRedirect = followRedirect
+                 followRedirect=True, redirectLimit=20,
+                 afterFoundGet=False):
+        self.followRedirect = followRedirect
         self.redirectLimit = redirectLimit
         self._redirectCount = 0
         self.timeout = timeout
         self.agent = agent
-
+        self.afterFoundGet = afterFoundGet
         if cookies is None:
             cookies = {}
         self.cookies = cookies
@@ -275,6 +309,8 @@ class HTTPClientFactory(protocol.ClientFactory):
 
     def buildProtocol(self, addr):
         p = protocol.ClientFactory.buildProtocol(self, addr)
+        p.followRedirect = self.followRedirect
+        p.afterFoundGet = self.afterFoundGet
         if self.timeout:
             timeoutCall = reactor.callLater(self.timeout, p.timeout)
             self.deferred.addBoth(self._cancelTimeout, timeoutCall)
@@ -322,7 +358,9 @@ class HTTPDownloader(HTTPClientFactory):
 
     def __init__(self, url, fileOrName,
                  method='GET', postdata=None, headers=None,
-                 agent="Twisted client", supportPartial=0):
+                 agent="Twisted client", supportPartial=0,
+                 timeout=0, cookies=None, followRedirect=1,
+                 redirectLimit=20):
         self.requestedPartial = 0
         if isinstance(fileOrName, types.StringTypes):
             self.fileName = fileOrName
@@ -336,11 +374,14 @@ class HTTPDownloader(HTTPClientFactory):
                     headers["range"] = "bytes=%d-" % fileLength
         else:
             self.file = fileOrName
-        HTTPClientFactory.__init__(self, url, method=method, postdata=postdata, headers=headers, agent=agent)
-        self.deferred = defer.Deferred()
-        self.waiting = 1
+        HTTPClientFactory.__init__(
+            self, url, method=method, postdata=postdata, headers=headers,
+            agent=agent, timeout=timeout, cookies=cookies,
+            followRedirect=followRedirect, redirectLimit=redirectLimit)
+
 
     def gotHeaders(self, headers):
+        HTTPClientFactory.gotHeaders(self, headers)
         if self.requestedPartial:
             contentRange = headers.get("content-range", None)
             if not contentRange:
@@ -352,6 +393,7 @@ class HTTPDownloader(HTTPClientFactory):
                 # server is acting wierdly
                 self.requestedPartial = 0
 
+
     def openFile(self, partialContent):
         if partialContent:
             file = open(self.fileName, 'rb+')
@@ -368,7 +410,6 @@ class HTTPDownloader(HTTPClientFactory):
         if partialContent and not self.requestedPartial:
             raise ValueError, "we shouldn't get partial content response if we didn't want it!"
         if self.waiting:
-            self.waiting = 0
             try:
                 if not self.file:
                     self.file = self.openFile(partialContent)
@@ -386,7 +427,24 @@ class HTTPDownloader(HTTPClientFactory):
             self.file = None
             self.deferred.errback(failure.Failure())
 
+
+    def noPage(self, reason):
+        """
+        Close the storage file and errback the waiting L{Deferred} with the
+        given reason.
+        """
+        if self.waiting:
+            self.waiting = 0
+            if self.file:
+                try:
+                    self.file.close()
+                except:
+                    log.err(None, "Error closing HTTPDownloader file")
+            self.deferred.errback(reason)
+
+
     def pageEnd(self):
+        self.waiting = 0
         if not self.file:
             return
         try:
@@ -397,6 +455,7 @@ class HTTPDownloader(HTTPClientFactory):
         self.deferred.callback(self.value)
 
 
+
 def _parse(url, defaultPort=None):
     """
     Split the given URL into the scheme, host, port, and path.
@@ -414,18 +473,25 @@ def _parse(url, defaultPort=None):
     url = url.strip()
     parsed = http.urlparse(url)
     scheme = parsed[0]
-    path = urlunparse(('','')+parsed[2:])
+    path = urlunparse(('', '') + parsed[2:])
+
     if defaultPort is None:
         if scheme == 'https':
             defaultPort = 443
         else:
             defaultPort = 80
+
     host, port = parsed[1], defaultPort
     if ':' in host:
         host, port = host.split(':')
-        port = int(port)
-    if path == "":
-        path = "/"
+        try:
+            port = int(port)
+        except ValueError:
+            port = defaultPort
+
+    if path == '':
+        path = '/'
+
     return scheme, host, port, path
 
 
@@ -489,7 +555,90 @@ def downloadPage(url, file, contextFactory=None, *args, **kwargs):
         *args, **kwargs).deferred
 
 
+# The code which follows is based on the new HTTP client implementation.  It
+# should be significantly better than anything above, though it is not yet
+# feature equivalent.
+
+from twisted.internet.protocol import ClientCreator
+from twisted.web.error import SchemeNotSupported
+from twisted.web._newclient import ResponseDone, Request, HTTP11ClientProtocol
+from twisted.web._newclient import Response
+
+class Agent(object):
+    """
+    L{Agent} is a very basic HTTP client.  It supports I{HTTP} scheme URIs.  It
+    does not support persistent connections.
+
+    @ivar _reactor: The L{IReactorTCP} implementation which will be used to set
+        up connections over which to issue requests.
+
+    @since: 9.0
+    """
+    _protocol = HTTP11ClientProtocol
+
+    def __init__(self, reactor):
+        self._reactor = reactor
+
+
+    def request(self, method, uri, headers=None, bodyProducer=None):
+        """
+        Issue a new request.
+
+        @param method: The request method to send.
+        @type method: C{str}
+
+        @param uri: The request URI send.
+        @type uri: C{str}
+
+        @param headers: The request headers to send.  If no I{Host} header is
+            included, one will be added based on the request URI.
+        @type headers: L{Headers}
+
+        @param bodyProducer: An object which will produce the request body or,
+            if the request body is to be empty, L{None}.
+        @type bodyProducer: L{IBodyProducer} provider
+
+        @return: A L{Deferred} which fires with the result of the request (a
+            L{Response} instance), or fails if there is a problem setting up a
+            connection over which to issue the request.  It may also fail with
+            L{SchemeNotSupported} if the scheme of the given URI is not
+            supported.
+        @rtype: L{Deferred}
+        """
+        scheme, host, port, path = _parse(uri)
+        if scheme != 'http':
+            return defer.fail(SchemeNotSupported(
+                    "Unsupported scheme: %r" % (scheme,)))
+        cc = ClientCreator(self._reactor, self._protocol)
+        d = cc.connectTCP(host, port)
+        if headers is None:
+            headers = Headers()
+        if not headers.hasHeader('host'):
+            # This is a lot of copying.  It might be nice if there were a bit
+            # less.
+            headers = Headers(dict(headers.getAllRawHeaders()))
+            headers.addRawHeader(
+                'host', self._computeHostValue(scheme, host, port))
+        def cbConnected(proto):
+            return proto.request(Request(method, path, headers, bodyProducer))
+        d.addCallback(cbConnected)
+        return d
+
+
+    def _computeHostValue(self, scheme, host, port):
+        """
+        Compute the string to use for the value of the I{Host} header, based on
+        the given scheme, host name, and port number.
+        """
+        if port == 80:
+            return host
+        return '%s:%d' % (host, port)
+
+
+
 __all__ = [
     'PartialDownloadError',
     'HTTPPageGetter', 'HTTPPageDownloader', 'HTTPClientFactory', 'HTTPDownloader',
-    'getPage', 'downloadPage']
+    'getPage', 'downloadPage',
+
+    'ResponseDone', 'Response', 'Agent']
