@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.web.test.test_http -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -32,6 +32,7 @@ from zope.interface import implements
 
 # twisted imports
 from twisted.internet import interfaces, reactor, protocol, address
+from twisted.internet.defer import Deferred
 from twisted.protocols import policies, basic
 from twisted.python import log
 try: # try importing the fast, C version
@@ -192,7 +193,9 @@ def urlparse(url):
 
 
 def parse_qs(qs, keep_blank_values=0, strict_parsing=0, unquote=unquote):
-    """like cgi.parse_qs, only with custom unquote function"""
+    """
+    like cgi.parse_qs, only with custom unquote function
+    """
     d = {}
     items = [s2 for s1 in qs.split("&") for s2 in s1.split(";")]
     for item in items:
@@ -212,7 +215,9 @@ def parse_qs(qs, keep_blank_values=0, strict_parsing=0, unquote=unquote):
     return d
 
 def datetimeToString(msSinceEpoch=None):
-    """Convert seconds since epoch to HTTP datetime string."""
+    """
+    Convert seconds since epoch to HTTP datetime string.
+    """
     if msSinceEpoch == None:
         msSinceEpoch = time.time()
     year, month, day, hh, mm, ss, wd, y, z = time.gmtime(msSinceEpoch)
@@ -223,7 +228,9 @@ def datetimeToString(msSinceEpoch=None):
     return s
 
 def datetimeToLogString(msSinceEpoch=None):
-    """Convert seconds since epoch to log datetime string."""
+    """
+    Convert seconds since epoch to log datetime string.
+    """
     if msSinceEpoch == None:
         msSinceEpoch = time.time()
     year, month, day, hh, mm, ss, wd, y, z = time.gmtime(msSinceEpoch)
@@ -260,9 +267,12 @@ def _logDateTimeStop():
         _resetLogDateTimeID.cancel()
 
 def timegm(year, month, day, hour, minute, second):
-    """Convert time tuple in GMT to seconds since epoch, GMT"""
+    """
+    Convert time tuple in GMT to seconds since epoch, GMT
+    """
     EPOCH = 1970
-    assert year >= EPOCH
+    if year < EPOCH:
+        raise ValueError("Years prior to %d not supported" % (EPOCH,))
     assert 1 <= month <= 12
     days = 365*(year-EPOCH) + calendar.leapdays(EPOCH, year)
     for i in range(1, month):
@@ -276,7 +286,9 @@ def timegm(year, month, day, hour, minute, second):
     return seconds
 
 def stringToDatetime(dateString):
-    """Convert an HTTP date string (one of three formats) to seconds since epoch."""
+    """
+    Convert an HTTP date string (one of three formats) to seconds since epoch.
+    """
     parts = dateString.split()
 
     if not parts[0][0:3].lower() in weekdayname_lower:
@@ -326,13 +338,16 @@ def stringToDatetime(dateString):
     return int(timegm(year, month, day, hour, min, sec))
 
 def toChunk(data):
-    """Convert string to a chunk.
+    """
+    Convert string to a chunk.
 
-    @returns: a tuple of strings representing the chunked encoding of data"""
+    @returns: a tuple of strings representing the chunked encoding of data
+    """
     return ("%x\r\n" % len(data), data, "\r\n")
 
 def fromChunk(data):
-    """Convert chunk to string.
+    """
+    Convert chunk to string.
 
     @returns: tuple (result, remaining), may raise ValueError.
     """
@@ -346,7 +361,8 @@ def fromChunk(data):
 
 
 def parseContentRange(header):
-    """Parse a content-range header into (start, end, realLength).
+    """
+    Parse a content-range header into (start, end, realLength).
 
     realLength might be None if real length is not known ('*').
     """
@@ -362,6 +378,7 @@ def parseContentRange(header):
     return (start, end, realLength)
 
 
+
 class StringTransport:
     """
     I am a StringIO wrapper that conforms for the transport API. I support
@@ -376,16 +393,29 @@ class StringTransport:
 
 
 class HTTPClient(basic.LineReceiver):
-    """A client for HTTP 1.0
+    """
+    A client for HTTP 1.0.
 
     Notes:
-    You probably want to send a 'Host' header with the name of
-    the site you're connecting to, in order to not break name
-    based virtual hosting.
+    You probably want to send a 'Host' header with the name of the site you're
+    connecting to, in order to not break name based virtual hosting.
+
+    @ivar length: The length of the request body in bytes.
+    @type length: C{int}
+
+    @ivar firstLine: Are we waiting for the first header line?
+    @type firstLine: C{bool}
+
+    @ivar __buffer: The buffer that stores the response to the HTTP request.
+    @type __buffer: A C{StringIO} object.
+
+    @ivar _header: Part or all of an HTTP request header.
+    @type _header: C{str}
     """
     length = None
-    firstLine = 1
+    firstLine = True
     __buffer = None
+    _header = ""
 
     def sendCommand(self, command, path):
         self.transport.write('%s %s HTTP/1.0\r\n' % (command, path))
@@ -396,9 +426,33 @@ class HTTPClient(basic.LineReceiver):
     def endHeaders(self):
         self.transport.write('\r\n')
 
+
+    def extractHeader(self, header):
+        """
+        Given a complete HTTP header, extract the field name and value and
+        process the header.
+
+        @param header: a complete HTTP request header of the form
+            'field-name: value'.
+        @type header: C{str}
+        """
+        key, val = header.split(':', 1)
+        val = val.lstrip()
+        self.handleHeader(key, val)
+        if key.lower() == 'content-length':
+            self.length = int(val)
+
+
     def lineReceived(self, line):
+        """
+        Parse the status line and headers for an HTTP request.
+
+        @param line: Part of an HTTP request header. Request bodies are parsed
+            in L{rawDataReceived}.
+        @type line: C{str}
+        """
         if self.firstLine:
-            self.firstLine = 0
+            self.firstLine = False
             l = line.split(None, 2)
             version = l[0]
             status = l[1]
@@ -409,21 +463,37 @@ class HTTPClient(basic.LineReceiver):
                 message = ""
             self.handleStatus(version, status, message)
             return
-        if line:
-            key, val = line.split(':', 1)
-            val = val.lstrip()
-            self.handleHeader(key, val)
-            if key.lower() == 'content-length':
-                self.length = int(val)
-        else:
+        if not line:
+            if self._header != "":
+                # Only extract headers if there are any
+                self.extractHeader(self._header)
             self.__buffer = StringIO()
             self.handleEndHeaders()
             self.setRawMode()
+            return
+
+        if line.startswith('\t') or line.startswith(' '):
+            # This line is part of a multiline header. According to RFC 822, in
+            # "unfolding" multiline headers you do not strip the leading
+            # whitespace on the continuing line.
+            self._header = self._header + line
+        elif self._header:
+            # This line starts a new header, so process the previous one.
+            self.extractHeader(self._header)
+            self._header = line
+        else: # First header
+            self._header = line
+
 
     def connectionLost(self, reason):
         self.handleResponseEnd()
 
     def handleResponseEnd(self):
+        """
+        The response has been completely received.
+
+        This callback may be invoked more than once per request.
+        """
         if self.__buffer is not None:
             b = self.__buffer.getvalue()
             self.__buffer = None
@@ -502,6 +572,16 @@ class Request:
         C{responseHeaders} instead.  C{headers} behaves mostly like a C{dict}
         and does not provide access to all header values nor does it allow
         multiple values for one header to be set.
+
+    @ivar notifications: A C{list} of L{Deferred}s which are waiting for
+        notification that the response to this request has been finished
+        (successfully or with an error).  Don't use this attribute directly,
+        instead use the L{Request.notifyFinish} method.
+
+    @ivar _disconnected: A flag which is C{False} until the connection over
+        which this request was received is closed and which is C{True} after
+        that.
+    @type _disconnected: C{bool}
     """
     implements(interfaces.IConsumer)
 
@@ -517,7 +597,11 @@ class Request:
     sentLength = 0 # content-length of response, or total bytes sent via chunking
     etag = None
     lastModified = None
+    args = None
+    path = None
+    content = None
     _forceSSL = 0
+    _disconnected = False
 
     def __init__(self, channel, queued):
         """
@@ -525,6 +609,7 @@ class Request:
         @param queued: are we in the request queue, or can we start writing to
             the transport?
         """
+        self.notifications = []
         self.channel = channel
         self.queued = queued
         self.requestHeaders = Headers()
@@ -563,7 +648,9 @@ class Request:
 
 
     def _cleanup(self):
-        """Called when have finished responding and are no longer queued."""
+        """
+        Called when have finished responding and are no longer queued.
+        """
         if self.producer:
             log.err(RuntimeError("Producer was not unregistered for %s" % self.uri))
             self.unregisterProducer()
@@ -575,11 +662,15 @@ class Request:
             # win32 suckiness, no idea why it does this
             pass
         del self.content
+        for d in self.notifications:
+            d.callback(None)
+        self.notifications = []
 
     # methods for channel - end users should not use these
 
     def noLongerQueued(self):
-        """Notify the object that it is no longer queued.
+        """
+        Notify the object that it is no longer queued.
 
         We start writing whatever data we have to the transport, etc.
 
@@ -643,7 +734,8 @@ class Request:
 
 
     def handleContentChunk(self, data):
-        """Write a chunk of data.
+        """
+        Write a chunk of data.
 
         This method is not intended for users.
         """
@@ -710,6 +802,7 @@ class Request:
                         self.channel.transport.loseConnection()
                         return
                     raise
+            self.content.seek(0, 0)
 
         self.process()
 
@@ -718,7 +811,8 @@ class Request:
         return '<%s %s %s>'% (self.method, self.uri, self.clientproto)
 
     def process(self):
-        """Override in subclasses.
+        """
+        Override in subclasses.
 
         This method is not intended for users.
         """
@@ -728,7 +822,9 @@ class Request:
     # consumer interface
 
     def registerProducer(self, producer, streaming):
-        """Register a producer."""
+        """
+        Register a producer.
+        """
         if self.producer:
             raise ValueError, "registering producer %s before previous one (%s) was unregistered" % (producer, self.producer)
 
@@ -736,12 +832,15 @@ class Request:
         self.producer = producer
 
         if self.queued:
-            producer.pauseProducing()
+            if streaming:
+                producer.pauseProducing()
         else:
             self.transport.registerProducer(producer, streaming)
 
     def unregisterProducer(self):
-        """Unregister the producer."""
+        """
+        Unregister the producer.
+        """
         if not self.queued:
             self.transport.unregisterProducer()
         self.producer = None
@@ -771,13 +870,35 @@ class Request:
 
 
     def getCookie(self, key):
-        """Get a cookie that was sent from the network.
+        """
+        Get a cookie that was sent from the network.
         """
         return self.received_cookies.get(key)
 
 
+    def notifyFinish(self):
+        """
+        Notify when the response to this request has finished.
+
+        @rtype: L{Deferred}
+
+        @return: A L{Deferred} which will be triggered when the request is
+            finished -- with a C{None} value if the request finishes
+            successfully or with an error if the request is interrupted by an
+            error (for example, the client closing the connection prematurely).
+        """
+        self.notifications.append(Deferred())
+        return self.notifications[-1]
+
+
     def finish(self):
-        """We are finished writing data."""
+        """
+        Indicate that all response data has been written to this L{Request}.
+        """
+        if self._disconnected:
+            raise RuntimeError(
+                "Request.finish called on a request after its connection was lost; "
+                "use Request.notifyFinish to keep track of this.")
         if self.finished:
             warnings.warn("Warning! request.finish called twice.", stacklevel=2)
             return
@@ -862,11 +983,12 @@ class Request:
                 self.transport.write(data)
 
     def addCookie(self, k, v, expires=None, domain=None, path=None, max_age=None, comment=None, secure=None):
-        """Set an outgoing HTTP cookie.
+        """
+        Set an outgoing HTTP cookie.
 
         In general, you should consider using sessions instead of cookies, see
-        twisted.web.server.Request.getSession and the
-        twisted.web.server.Session class for details.
+        L{twisted.web.server.Request.getSession} and the
+        L{twisted.web.server.Session} class for details.
         """
         cookie = '%s=%s' % (k, v)
         if expires is not None:
@@ -884,7 +1006,8 @@ class Request:
         self.cookies.append(cookie)
 
     def setResponseCode(self, code, message=None):
-        """Set the HTTP response code.
+        """
+        Set the HTTP response code.
         """
         if not isinstance(code, (int, long)):
             raise TypeError("HTTP response code must be int or long")
@@ -910,7 +1033,8 @@ class Request:
 
 
     def redirect(self, url):
-        """Utility function that does a redirect.
+        """
+        Utility function that does a redirect.
 
         The request should have finish() called after this.
         """
@@ -943,10 +1067,14 @@ class Request:
         if (not self.lastModified) or (self.lastModified < when):
             self.lastModified = when
 
-        modified_since = self.getHeader('if-modified-since')
-        if modified_since:
-            modified_since = stringToDatetime(modified_since.split(';', 1)[0])
-            if modified_since >= when:
+        modifiedSince = self.getHeader('if-modified-since')
+        if modifiedSince:
+            firstPart = modifiedSince.split(';', 1)[0]
+            try:
+                modifiedSince = stringToDatetime(firstPart)
+            except ValueError:
+                return None
+            if modifiedSince >= when:
                 self.setResponseCode(NOT_MODIFIED)
                 return CACHED
         return None
@@ -1017,7 +1145,8 @@ class Request:
 
 
     def getHost(self):
-        """Get my originally requesting transport's host.
+        """
+        Get my originally requesting transport's host.
 
         Don't rely on the 'transport' attribute, since Request objects may be
         copied remotely.  For information on this method's return value, see
@@ -1148,20 +1277,129 @@ class Request:
                 return name
         return names[0]
 
+
     def connectionLost(self, reason):
-        """connection was lost"""
-        pass
+        """
+        There is no longer a connection for this request to respond over.
+        Clean up anything which can't be useful anymore.
+        """
+        self._disconnected = True
+        self.channel = None
+        if self.content is not None:
+            self.content.close()
+        for d in self.notifications:
+            d.errback(reason)
+        self.notifications = []
 
 
 
-class _ChunkedTransferEncoding(object):
+class _DataLoss(Exception):
     """
-    Protocol for decoding chunked Transfer-Encoding, as defined by RFC 2616,
+    L{_DataLoss} indicates that not all of a message body was received. This
+    is only one of several possible exceptions which may indicate that data
+    was lost.  Because of this, it should not be checked for by
+    specifically; any unexpected exception should be treated as having
+    caused data loss.
+    """
+
+
+
+class PotentialDataLoss(Exception):
+    """
+    L{PotentialDataLoss} may be raised by a transfer encoding decoder's
+    C{noMoreData} method to indicate that it cannot be determined if the
+    entire response body has been delivered.  This only occurs when making
+    requests to HTTP servers which do not set I{Content-Length} or a
+    I{Transfer-Encoding} in the response because in this case the end of the
+    response is indicated by the connection being closed, an event which may
+    also be due to a transient network problem or other error.
+    """
+
+
+
+class _IdentityTransferDecoder(object):
+    """
+    Protocol for accumulating bytes up to a specified length.  This handles the
+    case where no I{Transfer-Encoding} is specified.
+
+    @ivar contentLength: Counter keeping track of how many more bytes there are
+        to receive.
+
+    @ivar dataCallback: A one-argument callable which will be invoked each
+        time application data is received.
+
+    @ivar finishCallback: A one-argument callable which will be invoked when
+        the terminal chunk is received.  It will be invoked with all bytes
+        which were delivered to this protocol which came after the terminal
+        chunk.
+    """
+    def __init__(self, contentLength, dataCallback, finishCallback):
+        self.contentLength = contentLength
+        self.dataCallback = dataCallback
+        self.finishCallback = finishCallback
+
+
+    def dataReceived(self, data):
+        """
+        Interpret the next chunk of bytes received.  Either deliver them to the
+        data callback or invoke the finish callback if enough bytes have been
+        received.
+
+        @raise RuntimeError: If the finish callback has already been invoked
+            during a previous call to this methood.
+        """
+        if self.dataCallback is None:
+            raise RuntimeError(
+                "_IdentityTransferDecoder cannot decode data after finishing")
+
+        if self.contentLength is None:
+            self.dataCallback(data)
+        elif len(data) < self.contentLength:
+            self.contentLength -= len(data)
+            self.dataCallback(data)
+        else:
+            # Make the state consistent before invoking any code belonging to
+            # anyone else in case noMoreData ends up being called beneath this
+            # stack frame.
+            contentLength = self.contentLength
+            dataCallback = self.dataCallback
+            finishCallback = self.finishCallback
+            self.dataCallback = self.finishCallback = None
+            self.contentLength = 0
+
+            dataCallback(data[:contentLength])
+            finishCallback(data[contentLength:])
+
+
+    def noMoreData(self):
+        """
+        All data which will be delivered to this decoder has been.  Check to
+        make sure as much data as was expected has been received.
+
+        @raise PotentialDataLoss: If the content length is unknown.
+        @raise _DataLoss: If the content length is known and fewer than that
+            many bytes have been delivered.
+
+        @return: C{None}
+        """
+        finishCallback = self.finishCallback
+        self.dataCallback = self.finishCallback = None
+        if self.contentLength is None:
+            finishCallback('')
+            raise PotentialDataLoss()
+        elif self.contentLength != 0:
+            raise _DataLoss()
+
+
+
+class _ChunkedTransferDecoder(object):
+    """
+    Protocol for decoding I{chunked} Transfer-Encoding, as defined by RFC 2616,
     section 3.6.1.  This protocol can interpret the contents of a request or
     response body which uses the I{chunked} Transfer-Encoding.  It cannot
     interpret any of the rest of the HTTP protocol.
 
-    It may make sense for _ChunkedTransferEncoding to be an actual IProtocol
+    It may make sense for _ChunkedTransferDecoder to be an actual IProtocol
     implementation.  Currently, the only user of this class will only ever
     call dataReceived on it.  However, it might be an improvement if the
     user could connect this to a transport and deliver connection lost
@@ -1229,8 +1467,8 @@ class _ChunkedTransferEncoding(object):
                 if data.startswith('\r\n'):
                     data = data[2:]
                     if self.finish:
-                        self.finishCallback(data)
                         self.state = 'finished'
+                        self.finishCallback(data)
                         data = ''
                     else:
                         self.state = 'chunk-length'
@@ -1248,10 +1486,20 @@ class _ChunkedTransferEncoding(object):
                     data = ''
             elif self.state == 'finished':
                 raise RuntimeError(
-                    "_ChunkedTransferEncoding.dataReceived called after last "
+                    "_ChunkedTransferDecoder.dataReceived called after last "
                     "chunk was processed")
 
 
+    def noMoreData(self):
+        """
+        Verify that all data has been received.  If it has not been, raise
+        L{_DataLoss}.
+        """
+        if self.state != 'finished':
+            raise _DataLoss(
+                "Chunked decoder in %r state, still expecting more data to "
+                "get to finished state." % (self.state,))
+
 
 
 class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
@@ -1259,7 +1507,7 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
     A receiver for HTTP requests.
 
     @ivar _transferDecoder: C{None} or an instance of
-        L{_ChunkedTransferEncoding} if the request body uses the I{chunked}
+        L{_ChunkedTransferDecoder} if the request body uses the I{chunked}
         Transfer-Encoding.
     """
 
@@ -1352,9 +1600,11 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
         data = data.strip()
         if header == 'content-length':
             self.length = int(data)
+            self._transferDecoder = _IdentityTransferDecoder(
+                self.length, self.requests[-1].handleContentChunk, self._finishRequestBody)
         elif header == 'transfer-encoding' and data.lower() == 'chunked':
             self.length = None
-            self._transferDecoder = _ChunkedTransferEncoding(
+            self._transferDecoder = _ChunkedTransferDecoder(
                 self.requests[-1].handleContentChunk, self._finishRequestBody)
 
         reqHeaders = self.requests[-1].requestHeaders
@@ -1392,14 +1642,7 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
 
     def rawDataReceived(self, data):
         self.resetTimeout()
-        if self._transferDecoder is not None:
-            self._transferDecoder.dataReceived(data)
-        elif len(data) < self.length:
-            self.requests[-1].handleContentChunk(data)
-            self.length = self.length - len(data)
-        else:
-            self.requests[-1].handleContentChunk(data[:self.length])
-            self._finishRequestBody(data[self.length:])
+        self._transferDecoder.dataReceived(data)
 
 
     def allHeadersReceived(self):
@@ -1456,7 +1699,9 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
 
 
     def requestDone(self, request):
-        """Called by first request in queue when it is done."""
+        """
+        Called by first request in queue when it is done.
+        """
         if request != self.requests[0]: raise TypeError
         del self.requests[0]
 
@@ -1481,7 +1726,9 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin):
 
 
 class HTTPFactory(protocol.ServerFactory):
-    """Factory for HTTP server."""
+    """
+    Factory for HTTP server.
+    """
 
     protocol = HTTPChannel
 
@@ -1517,7 +1764,9 @@ class HTTPFactory(protocol.ServerFactory):
         _logDateTimeStop()
 
     def _openLogFile(self, path):
-        """Override in subclasses, e.g. to use twisted.python.logfile."""
+        """
+        Override in subclasses, e.g. to use twisted.python.logfile.
+        """
         f = open(path, "a", 1)
         return f
 
@@ -1530,7 +1779,9 @@ class HTTPFactory(protocol.ServerFactory):
         return r[1:-1]
 
     def log(self, request):
-        """Log a request's result to the logfile, by default in combined log format."""
+        """
+        Log a request's result to the logfile, by default in combined log format.
+        """
         if hasattr(self, "logFile"):
             line = '%s - - %s "%s" %d %s "%s" "%s"\n' % (
                 request.getClientIP(),
