@@ -157,37 +157,89 @@ class Node(object):
         else:
             return 0
 
+
     def appendChild(self, child):
-        assert isinstance(child, Node)
+        """
+        Make the given L{Node} the last child of this node.
+
+        @param child: The L{Node} which will become a child of this node.
+
+        @raise TypeError: If C{child} is not a C{Node} instance.
+        """
+        if not isinstance(child, Node):
+            raise TypeError("expected Node instance")
         self.childNodes.append(child)
         child.parentNode = self
 
+
     def insertBefore(self, new, ref):
+        """
+        Make the given L{Node} C{new} a child of this node which comes before
+        the L{Node} C{ref}.
+
+        @param new: A L{Node} which will become a child of this node.
+
+        @param ref: A L{Node} which is already a child of this node which
+            C{new} will be inserted before.
+
+        @raise TypeError: If C{new} or C{ref} is not a C{Node} instance.
+
+        @return: C{new}
+        """
+        if not isinstance(new, Node) or not isinstance(ref, Node):
+            raise TypeError("expected Node instance")
         i = self.childNodes.index(ref)
         new.parentNode = self
         self.childNodes.insert(i, new)
         return new
 
+
     def removeChild(self, child):
+        """
+        Remove the given L{Node} from this node's children.
+
+        @param child: A L{Node} which is a child of this node which will no
+            longer be a child of this node after this method is called.
+
+        @raise TypeError: If C{child} is not a C{Node} instance.
+
+        @return: C{child}
+        """
+        if not isinstance(child, Node):
+            raise TypeError("expected Node instance")
         if child in self.childNodes:
             self.childNodes.remove(child)
             child.parentNode = None
         return child
 
     def replaceChild(self, newChild, oldChild):
-        assert isinstance(newChild, Node)
-        #if newChild.parentNode:
-        #    newChild.parentNode.removeChild(newChild)
-        assert (oldChild.parentNode is self,
-                ('oldChild (%s): oldChild.parentNode (%s) != self (%s)'
-                 % (oldChild, oldChild.parentNode, self)))
+        """
+        Replace a L{Node} which is already a child of this node with a
+        different node.
+
+        @param newChild: A L{Node} which will be made a child of this node.
+
+        @param oldChild: A L{Node} which is a child of this node which will
+            give up its position to C{newChild}.
+
+        @raise TypeError: If C{newChild} or C{oldChild} is not a C{Node}
+            instance.
+
+        @raise ValueError: If C{oldChild} is not a child of this C{Node}.
+        """
+        if not isinstance(newChild, Node) or not isinstance(oldChild, Node):
+            raise TypeError("expected Node instance")
+        if oldChild.parentNode is not self:
+            raise ValueError("oldChild is not a child of this node")
         self.childNodes[self.childNodes.index(oldChild)] = newChild
         oldChild.parentNode = None
         newChild.parentNode = self
 
+
     def lastChild(self):
         return self.childNodes[-1]
 
+
     def firstChild(self):
         if len(self.childNodes):
             return self.childNodes[0]
@@ -232,9 +284,18 @@ class Document(Node):
         return self.childNodes[0]
     documentElement=property(get_documentElement)
 
-    def appendChild(self, c):
-        assert not self.childNodes, "Only one element per document."
-        Node.appendChild(self, c)
+    def appendChild(self, child):
+        """
+        Make the given L{Node} the I{document element} of this L{Document}.
+
+        @param child: The L{Node} to make into this L{Document}'s document
+            element.
+
+        @raise ValueError: If this document already has a document element.
+        """
+        if self.childNodes:
+            raise ValueError("Only one element per document.")
+        Node.appendChild(self, child)
 
     def writexml(self, stream, indent='', addindent='', newl='', strip=0,
                  nsprefixes={}, namespace=''):
@@ -481,8 +542,26 @@ class Element(Node):
     def hasAttribute(self, name):
         return name in self.attributes
 
+
     def writexml(self, stream, indent='', addindent='', newl='', strip=0,
                  nsprefixes={}, namespace=''):
+        """
+        Serialize this L{Element} to the given stream.
+
+        @param stream: A file-like object to which this L{Element} will be
+            written.
+
+        @param nsprefixes: A C{dict} mapping namespace URIs as C{str} to
+            prefixes as C{str}.  This defines the prefixes which are already in
+            scope in the document at the point at which this L{Element} exists.
+            This is essentially an implementation detail for namespace support.
+            Applications should not try to use it.
+
+        @param namespace: The namespace URI as a C{str} which is the default at
+            the point in the document at which this L{Element} exists.  This is
+            essentially an implementation detail for namespace support.
+            Applications should not try to use it.
+        """
         # write beginning
         ALLOWSINGLETON = ('img', 'br', 'hr', 'base', 'meta', 'link', 'param',
                           'area', 'input', 'col', 'basefont', 'isindex',
@@ -512,15 +591,39 @@ class Element(Node):
             begin = [newl, indent] + begin
         bext = begin.extend
         writeattr = lambda _atr, _val: bext((' ', _atr, '="', escape(_val), '"'))
+
+        # Make a local for tracking what end tag will be used.  If namespace
+        # prefixes are involved, this will be changed to account for that
+        # before it's actually used.
+        endTagName = self.endTagName
+
         if namespace != self.namespace and self.namespace is not None:
-            if nsprefixes.has_key(self.namespace):
+            # If the current default namespace is not the namespace of this tag
+            # (and this tag has a namespace at all) then we'll write out
+            # something related to namespaces.
+            if self.namespace in nsprefixes:
+                # This tag's namespace already has a prefix bound to it.  Use
+                # that prefix.
                 prefix = nsprefixes[self.namespace]
-                bext(prefix+':'+self.tagName)
+                bext(prefix + ':' + self.tagName)
+                # Also make sure we use it for the end tag.
+                endTagName = prefix + ':' + self.endTagName
             else:
+                # This tag's namespace has no prefix bound to it.  Change the
+                # default namespace to this tag's namespace so we don't need
+                # prefixes.  Alternatively, we could add a new prefix binding.
+                # I'm not sure why the code was written one way rather than the
+                # other. -exarkun
                 bext(self.tagName)
                 writeattr("xmlns", self.namespace)
+                # The default namespace just changed.  Make sure any children
+                # know about this.
+                namespace = self.namespace
         else:
+            # This tag has no namespace or its namespace is already the default
+            # namespace.  Nothing extra to do here.
             bext(self.tagName)
+
         j = ''.join
         for attr, val in self.attributes.iteritems():
             if isinstance(attr, tuple):
@@ -552,16 +655,16 @@ class Element(Node):
                    self.tagName in FORMATNICELY:
                     w(j((newl, newindent)))
                 child.writexml(stream, newindent, addindent, newl, strip,
-                               downprefixes, self.namespace)
+                               downprefixes, namespace)
             if self.tagName in BLOCKELEMENTS:
                 w(j((newl, indent)))
-            w(j(("</", self.endTagName, '>')))
-
+            w(j(('</', endTagName, '>')))
         elif self.tagName.lower() not in ALLOWSINGLETON:
-            w(j(('></', self.endTagName, '>')))
+            w(j(('></', endTagName, '>')))
         else:
             w(" />")
 
+
     def __repr__(self):
         rep = "Element(%s" % repr(self.nodeName)
         if self.attributes:
