@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,11 +9,17 @@ from urlparse import urlparse, urlunsplit, clear_cache
 import random, urllib, cgi
 
 from twisted.python.compat import set
+from twisted.python.failure import Failure
 from twisted.trial import unittest
+from twisted.trial.unittest import TestCase
 from twisted.web import http, http_headers
+from twisted.web.http import PotentialDataLoss, _DataLoss
+from twisted.web.http import _IdentityTransferDecoder
 from twisted.protocols import loopback
 from twisted.internet.task import Clock
+from twisted.internet.error import ConnectionLost
 from twisted.test.proto_helpers import StringTransport
+from twisted.test.test_internet import DummyProducer
 from twisted.web.test.test_web import DummyChannel
 
 
@@ -298,19 +304,159 @@ class PersistenceTestCase(unittest.TestCase):
 
 
 
+class IdentityTransferEncodingTests(TestCase):
+    """
+    Tests for L{_IdentityTransferDecoder}.
+    """
+    def setUp(self):
+        """
+        Create an L{_IdentityTransferDecoder} with callbacks hooked up so that
+        calls to them can be inspected.
+        """
+        self.data = []
+        self.finish = []
+        self.contentLength = 10
+        self.decoder = _IdentityTransferDecoder(
+            self.contentLength, self.data.append, self.finish.append)
+
+
+    def test_exactAmountReceived(self):
+        """
+        If L{_IdentityTransferDecoder.dataReceived} is called with a string
+        with length equal to the content length passed to
+        L{_IdentityTransferDecoder}'s initializer, the data callback is invoked
+        with that string and the finish callback is invoked with a zero-length
+        string.
+        """
+        self.decoder.dataReceived('x' * self.contentLength)
+        self.assertEqual(self.data, ['x' * self.contentLength])
+        self.assertEqual(self.finish, [''])
+
+
+    def test_shortStrings(self):
+        """
+        If L{_IdentityTransferDecoder.dataReceived} is called multiple times
+        with strings which, when concatenated, are as long as the content
+        length provided, the data callback is invoked with each string and the
+        finish callback is invoked only after the second call.
+        """
+        self.decoder.dataReceived('x')
+        self.assertEqual(self.data, ['x'])
+        self.assertEqual(self.finish, [])
+        self.decoder.dataReceived('y' * (self.contentLength - 1))
+        self.assertEqual(self.data, ['x', 'y' * (self.contentLength - 1)])
+        self.assertEqual(self.finish, [''])
+
+
+    def test_longString(self):
+        """
+        If L{_IdentityTransferDecoder.dataReceived} is called with a string
+        with length greater than the provided content length, only the prefix
+        of that string up to the content length is passed to the data callback
+        and the remainder is passed to the finish callback.
+        """
+        self.decoder.dataReceived('x' * self.contentLength + 'y')
+        self.assertEqual(self.data, ['x' * self.contentLength])
+        self.assertEqual(self.finish, ['y'])
+
+
+    def test_rejectDataAfterFinished(self):
+        """
+        If data is passed to L{_IdentityTransferDecoder.dataReceived} after the
+        finish callback has been invoked, L{RuntimeError} is raised.
+        """
+        failures = []
+        def finish(bytes):
+            try:
+                decoder.dataReceived('foo')
+            except:
+                failures.append(Failure())
+        decoder = _IdentityTransferDecoder(5, self.data.append, finish)
+        decoder.dataReceived('x' * 4)
+        self.assertEqual(failures, [])
+        decoder.dataReceived('y')
+        failures[0].trap(RuntimeError)
+        self.assertEqual(
+            str(failures[0].value),
+            "_IdentityTransferDecoder cannot decode data after finishing")
+
+
+    def test_unknownContentLength(self):
+        """
+        If L{_IdentityTransferDecoder} is constructed with C{None} for the
+        content length, it passes all data delivered to it through to the data
+        callback.
+        """
+        data = []
+        finish = []
+        decoder = _IdentityTransferDecoder(None, data.append, finish.append)
+        decoder.dataReceived('x')
+        self.assertEqual(data, ['x'])
+        decoder.dataReceived('y')
+        self.assertEqual(data, ['x', 'y'])
+        self.assertEqual(finish, [])
+
+
+    def _verifyCallbacksUnreferenced(self, decoder):
+        """
+        Check the decoder's data and finish callbacks and make sure they are
+        None in order to help avoid references cycles.
+        """
+        self.assertIdentical(decoder.dataCallback, None)
+        self.assertIdentical(decoder.finishCallback, None)
+
+
+    def test_earlyConnectionLose(self):
+        """
+        L{_IdentityTransferDecoder.noMoreData} raises L{_DataLoss} if it is
+        called and the content length is known but not enough bytes have been
+        delivered.
+        """
+        self.decoder.dataReceived('x' * (self.contentLength - 1))
+        self.assertRaises(_DataLoss, self.decoder.noMoreData)
+        self._verifyCallbacksUnreferenced(self.decoder)
+
+
+    def test_unknownContentLengthConnectionLose(self):
+        """
+        L{_IdentityTransferDecoder.noMoreData} calls the finish callback and
+        raises L{PotentialDataLoss} if it is called and the content length is
+        unknown.
+        """
+        body = []
+        finished = []
+        decoder = _IdentityTransferDecoder(None, body.append, finished.append)
+        self.assertRaises(PotentialDataLoss, decoder.noMoreData)
+        self.assertEqual(body, [])
+        self.assertEqual(finished, [''])
+        self._verifyCallbacksUnreferenced(decoder)
+
+
+    def test_finishedConnectionLose(self):
+        """
+        L{_IdentityTransferDecoder.noMoreData} does not raise any exception if
+        it is called when the content length is known and that many bytes have
+        been delivered.
+        """
+        self.decoder.dataReceived('x' * self.contentLength)
+        self.decoder.noMoreData()
+        self._verifyCallbacksUnreferenced(self.decoder)
+
+
+
 class ChunkedTransferEncodingTests(unittest.TestCase):
     """
-    Tests for L{_ChunkedTransferEncoding}, which turns a byte stream encoded
+    Tests for L{_ChunkedTransferDecoder}, which turns a byte stream encoded
     using HTTP I{chunked} C{Transfer-Encoding} back into the original byte
     stream.
     """
     def test_decoding(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} decodes chunked-encoded data
+        L{_ChunkedTransferDecoder.dataReceived} decodes chunked-encoded data
         and passes the result to the specified callback.
         """
         L = []
-        p = http._ChunkedTransferEncoding(L.append, None)
+        p = http._ChunkedTransferDecoder(L.append, None)
         p.dataReceived('3\r\nabc\r\n5\r\n12345\r\n')
         p.dataReceived('a\r\n0123456789\r\n')
         self.assertEqual(L, ['abc', '12345', '0123456789'])
@@ -318,12 +464,12 @@ class ChunkedTransferEncodingTests(unittest.TestCase):
 
     def test_short(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} decodes chunks broken up and
+        L{_ChunkedTransferDecoder.dataReceived} decodes chunks broken up and
         delivered in multiple calls.
         """
         L = []
         finished = []
-        p = http._ChunkedTransferEncoding(L.append, finished.append)
+        p = http._ChunkedTransferDecoder(L.append, finished.append)
         for s in '3\r\nabc\r\n5\r\n12345\r\n0\r\n\r\n':
             p.dataReceived(s)
         self.assertEqual(L, ['a', 'b', 'c', '1', '2', '3', '4', '5'])
@@ -332,59 +478,103 @@ class ChunkedTransferEncodingTests(unittest.TestCase):
 
     def test_newlines(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} doesn't treat CR LF pairs
+        L{_ChunkedTransferDecoder.dataReceived} doesn't treat CR LF pairs
         embedded in chunk bodies specially.
         """
         L = []
-        p = http._ChunkedTransferEncoding(L.append, None)
+        p = http._ChunkedTransferDecoder(L.append, None)
         p.dataReceived('2\r\n\r\n\r\n')
         self.assertEqual(L, ['\r\n'])
 
 
     def test_extensions(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} disregards chunk-extension
+        L{_ChunkedTransferDecoder.dataReceived} disregards chunk-extension
         fields.
         """
         L = []
-        p = http._ChunkedTransferEncoding(L.append, None)
+        p = http._ChunkedTransferDecoder(L.append, None)
         p.dataReceived('3; x-foo=bar\r\nabc\r\n')
         self.assertEqual(L, ['abc'])
 
 
     def test_finish(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} interprets a zero-length
+        L{_ChunkedTransferDecoder.dataReceived} interprets a zero-length
         chunk as the end of the chunked data stream and calls the completion
         callback.
         """
         finished = []
-        p = http._ChunkedTransferEncoding(None, finished.append)
+        p = http._ChunkedTransferDecoder(None, finished.append)
         p.dataReceived('0\r\n\r\n')
         self.assertEqual(finished, [''])
 
 
     def test_extra(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} passes any bytes which come
+        L{_ChunkedTransferDecoder.dataReceived} passes any bytes which come
         after the terminating zero-length chunk to the completion callback.
         """
         finished = []
-        p = http._ChunkedTransferEncoding(None, finished.append)
+        p = http._ChunkedTransferDecoder(None, finished.append)
         p.dataReceived('0\r\n\r\nhello')
         self.assertEqual(finished, ['hello'])
 
 
     def test_afterFinished(self):
         """
-        L{_ChunkedTransferEncoding.dataReceived} raises L{RuntimeError} if it
+        L{_ChunkedTransferDecoder.dataReceived} raises L{RuntimeError} if it
         is called after it has seen the last chunk.
         """
-        p = http._ChunkedTransferEncoding(None, lambda bytes: None)
+        p = http._ChunkedTransferDecoder(None, lambda bytes: None)
         p.dataReceived('0\r\n\r\n')
         self.assertRaises(RuntimeError, p.dataReceived, 'hello')
 
 
+    def test_earlyConnectionLose(self):
+        """
+        L{_ChunkedTransferDecoder.noMoreData} raises L{_DataLoss} if it is
+        called and the end of the last trailer has not yet been received.
+        """
+        parser = http._ChunkedTransferDecoder(None, lambda bytes: None)
+        parser.dataReceived('0\r\n\r')
+        exc = self.assertRaises(_DataLoss, parser.noMoreData)
+        self.assertEqual(
+            str(exc),
+            "Chunked decoder in 'trailer' state, still expecting more data "
+            "to get to finished state.")
+
+
+    def test_finishedConnectionLose(self):
+        """
+        L{_ChunkedTransferDecoder.noMoreData} does not raise any exception if
+        it is called after the terminal zero length chunk is received.
+        """
+        parser = http._ChunkedTransferDecoder(None, lambda bytes: None)
+        parser.dataReceived('0\r\n\r\n')
+        parser.noMoreData()
+
+
+    def test_reentrantFinishedNoMoreData(self):
+        """
+        L{_ChunkedTransferDecoder.noMoreData} can be called from the finished
+        callback without raising an exception.
+        """
+        errors = []
+        successes = []
+        def finished(extra):
+            try:
+                parser.noMoreData()
+            except:
+                errors.append(Failure())
+            else:
+                successes.append(True)
+        parser = http._ChunkedTransferDecoder(None, finished)
+        parser.dataReceived('0\r\n\r\n')
+        self.assertEqual(errors, [])
+        self.assertEqual(successes, [True])
+
+
 
 class ChunkingTestCase(unittest.TestCase):
 
@@ -473,7 +663,7 @@ class ParsingTestCase(unittest.TestCase):
             "",
             ""]
 
-        channel = self.runRequest('\n'.join(requestLines), MyRequest, 0)
+        self.runRequest('\n'.join(requestLines), MyRequest, 0)
         [request] = processed
         self.assertEquals(
             request.requestHeaders.getRawHeaders('foo'), ['bar'])
@@ -604,7 +794,14 @@ GET /?key=value&multiple=two+words&multiple=more%20words&empty= HTTP/1.0
         self.runRequest(httpRequest, MyRequest)
 
 
-    def testPOST(self):
+    def test_formPOSTRequest(self):
+        """
+        The request body of a I{POST} request with a I{Content-Type} header
+        of I{application/x-www-form-urlencoded} is parsed according to that
+        content type and made available in the C{args} attribute of the
+        request object.  The original bytes of the request may still be read
+        from the C{content} attribute.
+        """
         query = 'key=value&multiple=two+words&multiple=more%20words&empty='
         httpRequest = '''\
 POST / HTTP/1.0
@@ -620,6 +817,10 @@ Content-Type: application/x-www-form-urlencoded
                 testcase.assertEquals(self.args["key"], ["value"])
                 testcase.assertEquals(self.args["empty"], [""])
                 testcase.assertEquals(self.args["multiple"], ["two words", "more words"])
+
+                # Reading from the content file-like must produce the entire
+                # request body.
+                testcase.assertEquals(self.content.read(), query)
                 testcase.didRequest = 1
                 self.finish()
 
@@ -1060,3 +1261,271 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
         req.parseCookies()
         self.assertEquals(req.received_cookies, {"test": '"lemur"',
                                                  "test2": '"panda"'})
+
+
+    def test_connectionLost(self):
+        """
+        L{http.Request.connectionLost} closes L{Request.content} and drops the
+        reference to the L{HTTPChannel} to assist with garbage collection.
+        """
+        req = http.Request(DummyChannel(), None)
+
+        # Cause Request.content to be created at all.
+        req.gotLength(10)
+
+        # Grab a reference to content in case the Request drops it later on.
+        content = req.content
+
+        # Put some bytes into it
+        req.handleContentChunk("hello")
+
+        # Then something goes wrong and content should get closed.
+        req.connectionLost(Failure(ConnectionLost("Finished")))
+        self.assertTrue(content.closed)
+        self.assertIdentical(req.channel, None)
+
+
+    def test_registerProducerTwiceFails(self):
+        """
+        Calling L{Request.registerProducer} when a producer is already
+        registered raises ValueError.
+        """
+        req = http.Request(DummyChannel(), None)
+        req.registerProducer(DummyProducer(), True)
+        self.assertRaises(
+            ValueError, req.registerProducer, DummyProducer(), True)
+
+
+    def test_registerProducerWhenQueuedPausesPushProducer(self):
+        """
+        Calling L{Request.registerProducer} with an IPushProducer when the
+        request is queued pauses the producer.
+        """
+        req = http.Request(DummyChannel(), True)
+        producer = DummyProducer()
+        req.registerProducer(producer, True)
+        self.assertEquals(['pause'], producer.events)
+
+
+    def test_registerProducerWhenQueuedDoesntPausePullProducer(self):
+        """
+        Calling L{Request.registerProducer} with an IPullProducer when the
+        request is queued does not pause the producer, because it doesn't make
+        sense to pause a pull producer.
+        """
+        req = http.Request(DummyChannel(), True)
+        producer = DummyProducer()
+        req.registerProducer(producer, False)
+        self.assertEquals([], producer.events)
+
+
+    def test_registerProducerWhenQueuedDoesntRegisterPushProducer(self):
+        """
+        Calling L{Request.registerProducer} with an IPushProducer when the
+        request is queued does not register the producer on the request's
+        transport.
+        """
+        self.assertIdentical(
+            None, getattr(http.StringTransport, 'registerProducer', None),
+            "StringTransport cannot implement registerProducer for this test "
+            "to be valid.")
+        req = http.Request(DummyChannel(), True)
+        producer = DummyProducer()
+        req.registerProducer(producer, True)
+        # This is a roundabout assertion: http.StringTransport doesn't
+        # implement registerProducer, so Request.registerProducer can't have
+        # tried to call registerProducer on the transport.
+        self.assertIsInstance(req.transport, http.StringTransport)
+
+
+    def test_registerProducerWhenQueuedDoesntRegisterPullProducer(self):
+        """
+        Calling L{Request.registerProducer} with an IPullProducer when the
+        request is queued does not register the producer on the request's
+        transport.
+        """
+        self.assertIdentical(
+            None, getattr(http.StringTransport, 'registerProducer', None),
+            "StringTransport cannot implement registerProducer for this test "
+            "to be valid.")
+        req = http.Request(DummyChannel(), True)
+        producer = DummyProducer()
+        req.registerProducer(producer, False)
+        # This is a roundabout assertion: http.StringTransport doesn't
+        # implement registerProducer, so Request.registerProducer can't have
+        # tried to call registerProducer on the transport.
+        self.assertIsInstance(req.transport, http.StringTransport)
+
+
+    def test_registerProducerWhenNotQueuedRegistersPushProducer(self):
+        """
+        Calling L{Request.registerProducer} with an IPushProducer when the
+        request is not queued registers the producer as a push producer on the
+        request's transport.
+        """
+        req = http.Request(DummyChannel(), False)
+        producer = DummyProducer()
+        req.registerProducer(producer, True)
+        self.assertEquals([(producer, True)], req.transport.producers)
+
+
+    def test_registerProducerWhenNotQueuedRegistersPullProducer(self):
+        """
+        Calling L{Request.registerProducer} with an IPullProducer when the
+        request is not queued registers the producer as a pull producer on the
+        request's transport.
+        """
+        req = http.Request(DummyChannel(), False)
+        producer = DummyProducer()
+        req.registerProducer(producer, False)
+        self.assertEquals([(producer, False)], req.transport.producers)
+
+
+    def test_connectionLostNotification(self):
+        """
+        L{Request.connectionLost} triggers all finish notification Deferreds
+        and cleans up per-request state.
+        """
+        d = DummyChannel()
+        request = http.Request(d, True)
+        finished = request.notifyFinish()
+        request.connectionLost(Failure(ConnectionLost("Connection done")))
+        self.assertIdentical(request.channel, None)
+        return self.assertFailure(finished, ConnectionLost)
+
+
+    def test_finishNotification(self):
+        """
+        L{Request.finish} triggers all finish notification Deferreds.
+        """
+        request = http.Request(DummyChannel(), False)
+        finished = request.notifyFinish()
+        # Force the request to have a non-None content attribute.  This is
+        # probably a bug in Request.
+        request.gotLength(1)
+        request.finish()
+        return finished
+
+
+    def test_finishAfterConnectionLost(self):
+        """
+        Calling L{Request.finish} after L{Request.connectionLost} has been
+        called results in a L{RuntimeError} being raised.
+        """
+        channel = DummyChannel()
+        transport = channel.transport
+        req = http.Request(channel, False)
+        req.connectionLost(Failure(ConnectionLost("The end.")))
+        self.assertRaises(RuntimeError, req.finish)
+
+
+
+class MultilineHeadersTestCase(unittest.TestCase):
+    """
+    Tests to exercise handling of multiline headers by L{HTTPClient}.  RFCs 1945
+    (HTTP 1.0) and 2616 (HTTP 1.1) state that HTTP message header fields can
+    span multiple lines if each extra line is preceded by at least one space or
+    horizontal tab.
+    """
+    def setUp(self):
+        """
+        Initialize variables used to verify that the header-processing functions
+        are getting called.
+        """
+        self.handleHeaderCalled = False
+        self.handleEndHeadersCalled = False
+
+    # Dictionary of sample complete HTTP header key/value pairs, including
+    # multiline headers.
+    expectedHeaders = {'Content-Length': '10',
+                       'X-Multiline' : 'line-0\tline-1',
+                       'X-Multiline2' : 'line-2 line-3'}
+
+    def ourHandleHeader(self, key, val):
+        """
+        Dummy implementation of L{HTTPClient.handleHeader}.
+        """
+        self.handleHeaderCalled = True
+        self.assertEquals(val, self.expectedHeaders[key])
+
+
+    def ourHandleEndHeaders(self):
+        """
+        Dummy implementation of L{HTTPClient.handleEndHeaders}.
+        """
+        self.handleEndHeadersCalled = True
+
+
+    def test_extractHeader(self):
+        """
+        A header isn't processed by L{HTTPClient.extractHeader} until it is
+        confirmed in L{HTTPClient.lineReceived} that the header has been
+        received completely.
+        """
+        c = ClientDriver()
+        c.handleHeader = self.ourHandleHeader
+        c.handleEndHeaders = self.ourHandleEndHeaders
+
+        c.lineReceived('HTTP/1.0 201')
+        c.lineReceived('Content-Length: 10')
+        self.assertIdentical(c.length, None)
+        self.assertFalse(self.handleHeaderCalled)
+        self.assertFalse(self.handleEndHeadersCalled)
+
+        # Signal end of headers.
+        c.lineReceived('')
+        self.assertTrue(self.handleHeaderCalled)
+        self.assertTrue(self.handleEndHeadersCalled)
+
+        self.assertEquals(c.length, 10)
+
+
+    def test_noHeaders(self):
+        """
+        An HTTP request with no headers will not cause any calls to
+        L{handleHeader} but will cause L{handleEndHeaders} to be called on
+        L{HTTPClient} subclasses.
+        """
+        c = ClientDriver()
+        c.handleHeader = self.ourHandleHeader
+        c.handleEndHeaders = self.ourHandleEndHeaders
+        c.lineReceived('HTTP/1.0 201')
+
+        # Signal end of headers.
+        c.lineReceived('')
+        self.assertFalse(self.handleHeaderCalled)
+        self.assertTrue(self.handleEndHeadersCalled)
+
+        self.assertEquals(c.version, 'HTTP/1.0')
+        self.assertEquals(c.status, '201')
+
+
+    def test_multilineHeaders(self):
+        """
+        L{HTTPClient} parses multiline headers by buffering header lines until
+        an empty line or a line that does not start with whitespace hits
+        lineReceived, confirming that the header has been received completely.
+        """
+        c = ClientDriver()
+        c.handleHeader = self.ourHandleHeader
+        c.handleEndHeaders = self.ourHandleEndHeaders
+
+        c.lineReceived('HTTP/1.0 201')
+        c.lineReceived('X-Multiline: line-0')
+        self.assertFalse(self.handleHeaderCalled)
+        # Start continuing line with a tab.
+        c.lineReceived('\tline-1')
+        c.lineReceived('X-Multiline2: line-2')
+        # The previous header must be complete, so now it can be processed.
+        self.assertTrue(self.handleHeaderCalled)
+        # Start continuing line with a space.
+        c.lineReceived(' line-3')
+        c.lineReceived('Content-Length: 10')
+
+        # Signal end of headers.
+        c.lineReceived('')
+        self.assertTrue(self.handleEndHeadersCalled)
+
+        self.assertEquals(c.version, 'HTTP/1.0')
+        self.assertEquals(c.status, '201')
+        self.assertEquals(c.length, 10)
