@@ -1,24 +1,15 @@
 # -*- test-case-name: twisted.web.test.test_xmlrpc -*-
-#
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 A generic resource for publishing objects via XML-RPC.
 
-Requires xmlrpclib (comes standard with Python 2.2 and later, otherwise can be
-downloaded from http://www.pythonware.com/products/xmlrpc/).
-
-Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
+Maintainer: Itamar Shtull-Trauring
 """
-from __future__ import nested_scopes
-
-__version__ = "$Revision: 1.32 $"[11:-2]
 
 # System Imports
-import xmlrpclib
-import urlparse
+import sys, xmlrpclib, urlparse
 
 # Sibling Imports
 from twisted.web import resource, server, http
@@ -36,6 +27,12 @@ Binary = xmlrpclib.Binary
 Boolean = xmlrpclib.Boolean
 DateTime = xmlrpclib.DateTime
 
+# On Python 2.4 and earlier, DateTime.decode returns unicode.
+if sys.version_info[:2] < (2, 5):
+    _decode = DateTime.decode
+    DateTime.decode = lambda self, value: _decode(self, value.encode('ascii'))
+
+
 class NoSuchFunction(Fault):
     """
     There is no function by the given name.
@@ -123,29 +120,34 @@ class XMLRPC(resource.Resource):
             except Fault, f:
                 self._cbRender(f, request)
             else:
-                defer.maybeDeferred(function, *args).addErrback(
-                    self._ebRender
-                ).addCallback(
-                    self._cbRender, request
-                )
+                d = defer.maybeDeferred(function, *args)
+                d.addErrback(self._ebRender)
+                d.addCallback(self._cbRender, request)
         return server.NOT_DONE_YET
 
+
     def _cbRender(self, result, request):
         if isinstance(result, Handler):
             result = result.result
         if not isinstance(result, Fault):
             result = (result,)
         try:
-            s = xmlrpclib.dumps(result, methodresponse=True,
-                                allow_none=self.allowNone)
-        except Exception, e:
-            f = Fault(self.FAILURE, "Can't serialize output: %s" % (e,))
-            s = xmlrpclib.dumps(f, methodresponse=True,
-                                allow_none=self.allowNone)
-        request.setHeader("content-length", str(len(s)))
-        request.write(s)
+            try:
+                content = xmlrpclib.dumps(
+                    result, methodresponse=True,
+                    allow_none=self.allowNone)
+            except Exception, e:
+                f = Fault(self.FAILURE, "Can't serialize output: %s" % (e,))
+                content = xmlrpclib.dumps(f, methodresponse=True,
+                                          allow_none=self.allowNone)
+
+            request.setHeader("content-length", str(len(content)))
+            request.write(content)
+        except:
+            log.err()
         request.finish()
 
+
     def _ebRender(self, failure):
         if isinstance(failure.value, Fault):
             return failure.value
@@ -312,13 +314,13 @@ class _QueryFactory(protocol.ClientFactory):
         if not self.deferred:
             return
         try:
-            response = xmlrpclib.loads(contents)
+            response = xmlrpclib.loads(contents)[0][0]
         except:
             deferred, self.deferred = self.deferred, None
             deferred.errback(failure.Failure())
         else:
             deferred, self.deferred = self.deferred, None
-            deferred.callback(response[0][0])
+            deferred.callback(response)
 
     def clientConnectionLost(self, _, reason):
         if self.deferred is not None:
@@ -399,6 +401,14 @@ class Proxy:
         self.allowNone = allowNone
 
     def callRemote(self, method, *args):
+        """
+        Call remote XML-RPC C{method} with given arguments.
+
+        @return: a L{defer.Deferred} that will fire with the method response,
+            or a failure if the method failed. Generally, the failure type will
+            be L{Fault}, but you can also have an C{IndexError} on some buggy
+            servers giving empty responses.
+        """
         factory = self.queryFactory(
             self.path, self.host, method, self.user,
             self.password, self.allowNone, args)
@@ -410,5 +420,8 @@ class Proxy:
             reactor.connectTCP(self.host, self.port or 80, factory)
         return factory.deferred
 
-__all__ = ["XMLRPC", "Handler", "NoSuchFunction", "Fault", "Proxy"]
 
+__all__ = [
+    "XMLRPC", "Handler", "NoSuchFunction", "Proxy",
+
+    "Fault", "Binary", "Boolean", "DateTime"]
